"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DidExchangeStateMachine = void 0;
const error_1 = require("../../error");
const messageType_1 = require("../../utils/messageType");
const messages_1 = require("./messages");
class DidExchangeStateMachine {
    static assertCreateMessageState(messageType, record) {
        const rule = this.createMessageStateRules.find((r) => messageType_1.canHandleMessageType(r.message, messageType));
        if (!rule) {
            throw new error_1.AriesFrameworkError(`Could not find create message rule for ${messageType}`);
        }
        if (rule.state !== record.state || rule.role !== record.role) {
            throw new error_1.AriesFrameworkError(`Record with role ${record.role} is in invalid state ${record.state} to create ${messageType}. Expected state for role ${rule.role} is ${rule.state}.`);
        }
    }
    static assertProcessMessageState(messageType, record) {
        const rule = this.processMessageStateRules.find((r) => messageType_1.canHandleMessageType(r.message, messageType));
        if (!rule) {
            throw new error_1.AriesFrameworkError(`Could not find create message rule for ${messageType}`);
        }
        if (rule.state !== record.state || rule.role !== record.role) {
            throw new error_1.AriesFrameworkError(`Record with role ${record.role} is in invalid state ${record.state} to process ${messageType}. Expected state for role ${rule.role} is ${rule.state}.`);
        }
    }
    static nextState(messageType, record) {
        const rule = this.createMessageStateRules
            .concat(this.processMessageStateRules)
            .find((r) => messageType_1.canHandleMessageType(r.message, messageType) && r.role === record.role);
        if (!rule) {
            throw new error_1.AriesFrameworkError(`Could not find create message rule for messageType ${messageType}, state ${record.state} and role ${record.role}`);
        }
        return rule.nextState;
    }
}
exports.DidExchangeStateMachine = DidExchangeStateMachine;
DidExchangeStateMachine.createMessageStateRules = [
    {
        message: messages_1.DidExchangeRequestMessage,
        state: "invitation-received" /* InvitationReceived */,
        role: "requester" /* Requester */,
        nextState: "request-sent" /* RequestSent */,
    },
    {
        message: messages_1.DidExchangeResponseMessage,
        state: "request-received" /* RequestReceived */,
        role: "responder" /* Responder */,
        nextState: "response-sent" /* ResponseSent */,
    },
    {
        message: messages_1.DidExchangeCompleteMessage,
        state: "response-received" /* ResponseReceived */,
        role: "requester" /* Requester */,
        nextState: "completed" /* Completed */,
    },
];
DidExchangeStateMachine.processMessageStateRules = [
    {
        message: messages_1.DidExchangeRequestMessage,
        state: "invitation-sent" /* InvitationSent */,
        role: "responder" /* Responder */,
        nextState: "request-received" /* RequestReceived */,
    },
    {
        message: messages_1.DidExchangeResponseMessage,
        state: "request-sent" /* RequestSent */,
        role: "requester" /* Requester */,
        nextState: "response-received" /* ResponseReceived */,
    },
    {
        message: messages_1.DidExchangeCompleteMessage,
        state: "response-sent" /* ResponseSent */,
        role: "responder" /* Responder */,
        nextState: "completed" /* Completed */,
    },
];
//# sourceMappingURL=DidExchangeStateMachine.js.map