"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rfc0160StateFromDidExchangeState = exports.ConnectionState = void 0;
/**
 * Connection states as defined in RFC 0160.
 *
 * @see https://github.com/hyperledger/aries-rfcs/blob/master/features/0160-connection-protocol/README.md#states
 */
var ConnectionState;
(function (ConnectionState) {
    ConnectionState["Null"] = "null";
    ConnectionState["Invited"] = "invited";
    ConnectionState["Requested"] = "requested";
    ConnectionState["Responded"] = "responded";
    ConnectionState["Complete"] = "complete";
})(ConnectionState = exports.ConnectionState || (exports.ConnectionState = {}));
function rfc0160StateFromDidExchangeState(didExchangeState) {
    const stateMapping = {
        ["start" /* Start */]: ConnectionState.Null,
        ["abandoned" /* Abandoned */]: ConnectionState.Null,
        ["invitation-received" /* InvitationReceived */]: ConnectionState.Invited,
        ["invitation-sent" /* InvitationSent */]: ConnectionState.Invited,
        ["request-received" /* RequestReceived */]: ConnectionState.Requested,
        ["request-sent" /* RequestSent */]: ConnectionState.Requested,
        ["response-received" /* ResponseReceived */]: ConnectionState.Responded,
        ["response-sent" /* ResponseSent */]: ConnectionState.Responded,
        ["completed" /* Completed */]: "completed" /* Completed */,
    };
    return stateMapping[didExchangeState];
}
exports.rfc0160StateFromDidExchangeState = rfc0160StateFromDidExchangeState;
//# sourceMappingURL=ConnectionState.js.map