"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V2RequestCredentialHandler = void 0;
const helpers_1 = require("../../../../../agent/helpers");
const AriesFrameworkError_1 = require("../../../../../error/AriesFrameworkError");
const V2OfferCredentialMessage_1 = require("../messages/V2OfferCredentialMessage");
const V2ProposeCredentialMessage_1 = require("../messages/V2ProposeCredentialMessage");
const V2RequestCredentialMessage_1 = require("../messages/V2RequestCredentialMessage");
class V2RequestCredentialHandler {
    constructor(credentialService, agentConfig, didCommMessageRepository) {
        this.supportedMessages = [V2RequestCredentialMessage_1.V2RequestCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.didCommMessageRepository = didCommMessageRepository;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processRequest(messageContext);
        const requestMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2RequestCredentialMessage_1.V2RequestCredentialMessage,
        });
        if (!requestMessage) {
            throw new AriesFrameworkError_1.AriesFrameworkError('Missing request message in V2RequestCredentialHandler');
        }
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
        });
        const proposeMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2ProposeCredentialMessage_1.V2ProposeCredentialMessage,
        });
        const shouldAutoRespond = this.credentialService.shouldAutoRespondToRequest(credentialRecord, requestMessage, proposeMessage !== null && proposeMessage !== void 0 ? proposeMessage : undefined, offerMessage !== null && offerMessage !== void 0 ? offerMessage : undefined);
        if (shouldAutoRespond) {
            return await this.createCredential(credentialRecord, messageContext, requestMessage, offerMessage);
        }
    }
    async createCredential(record, messageContext, requestMessage, offerMessage) {
        this.agentConfig.logger.info(`Automatically sending credential with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        const options = {
            comment: requestMessage.comment,
            autoAcceptCredential: record.autoAcceptCredential,
            credentialRecordId: record.id,
        };
        const { message, credentialRecord } = await this.credentialService.createCredential(record, options);
        if (messageContext.connection) {
            return helpers_1.createOutboundMessage(messageContext.connection, message);
        }
        else if (requestMessage.service && (offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.service)) {
            const recipientService = requestMessage.service;
            const ourService = offerMessage.service;
            // Set ~service, update message in record (for later use)
            message.setService(ourService);
            await this.credentialService.update(credentialRecord);
            return helpers_1.createOutboundServiceMessage({
                payload: message,
                service: recipientService.resolvedDidCommService,
                senderKey: ourService.resolvedDidCommService.recipientKeys[0],
            });
        }
        this.agentConfig.logger.error(`Could not automatically create credential request`);
    }
}
exports.V2RequestCredentialHandler = V2RequestCredentialHandler;
//# sourceMappingURL=V2RequestCredentialHandler.js.map